{% extends 'base.html' %}

{% block title %}
Home
{% endblock %}

{% block content %}
<p>Try searching for some manga title using the box above.</p>
<p>Logging in allows you to follow manga titles.</p>
{% endblock %}
